#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <GL/glut.h>
#include "project4.h"
#include "Matrix4.h"

using namespace std;

int Window::width  = 512;   // set window width in pixels here
int Window::height = 512;   // set window height in pixels here

//----------------------------------------------------------------------------
// Callback method called when window is resized.
void Window::reshapeCallback(int w, int h)
{
  width = w;
  height = h;
  glViewport(0, 0, w, h);  // set new viewport size
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-10.0, 10.0, -10.0, 10.0, 10, 1000.0); // set perspective projection viewing frustum

  /* moved camera position to modelview matrix */
  //glTranslatef(0, 0, -20);

}

//----------------------------------------------------------------------------
// Callback method called when window readraw is necessary or
// when glutPostRedisplay() was called.
void Window::displayCallback(void)
{
  Matrix4 sphere1,sphere2;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  // clear color and depth buffers
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  GLfloat params[4] = {1., 1., 1., 1.};
  glMaterialfv(GL_FRONT, GL_SPECULAR, params);

  /* set the camera position */
  glTranslatef(0, 0, -20);

  sphere1.makeTranslate(-8.0, 0, 0);
  glMultMatrixd(sphere1.getPointer());
  /* Switch to a finer tessellation for debugging to see the fixed function lighting
  ** be sure to turn this back to 12,12 before grading.
  glutSolidSphere(6.0, 120, 120);
  */
  glutSolidSphere(6.0, 12, 12); 
  
  // done with this object, clear the modelview matrix
  glLoadIdentity();

  /* set the camera position again since we just cleared everything */
  glTranslatef(0, 0, -20);

  sphere2.makeTranslate(8.0, 0, 0);
  glMultMatrixd(sphere2.getPointer());
  /* Switch to a finer tessellation for debugging to see the fixed function lighting
  ** be sure to turn this back to 12,12 before grading.
  glutSolidSphere(6.0, 120, 120);
  */
  glutSolidSphere(6.0, 12, 12); 

  
  glutSwapBuffers();
}

int main(int argc, char *argv[])
{
  GLfloat mat_specular[]   = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat mat_shininess[]  = { 50.0 };
  GLfloat light0_position[] = { 10.0, 10.0, 10.0, 0.0 };
  
  glutInit(&argc, argv);      	      	      // initialize GLUT
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);   // open an OpenGL context with double buffering, RGB colors, and depth buffering
  glutInitWindowSize(Window::width, Window::height);      // set initial window size
  glutCreateWindow("Two Spheres");    	                // open window and set window title

  glEnable(GL_DEPTH_TEST);            	      // enable depth buffering
  glClearColor(0.0, 0.0, 0.0, 0.0);   	      // set clear color to black
  glMatrixMode(GL_PROJECTION); 
  
  // Generate material properties:
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
  
  // Generate light source:
  glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  
  // Install callback functions:
  glutDisplayFunc(Window::displayCallback);
  glutReshapeFunc(Window::reshapeCallback);
    
  glutMainLoop();
  return 0;
}

